//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: Arena.cpp
//
// Description: Server-side Arena class which describes the arena where
//              the robots will live
//
//*****************************************************************************

#include "arena.h"
#include <time.h>

Arena::Arena(unsigned int width, unsigned int height)
: m_width( width )
, m_height( height )
{
    // Resize the vectors
    m_Squares.resize( m_width );

    for( unsigned int i=0; i<m_width; ++i )
    {
        m_Squares[i].resize( m_height );
    }

    // Create the Squares
    for(i=0; i<m_width; ++i )
    {
        for(unsigned int j=0; j<m_height; ++j )
        {
            if( (0 == i) || (0 == j) || (i==m_width-1) || (j==m_height-1) )
            {
                m_Squares[i][j] = Square( Square::Wall );
            }
            else
            {
                m_Squares[i][j] = Square( Square::Empty );
            }
        }
    }

    // Assign the neighbours
    for(i=0; i<m_width; ++i )
    {
        for(unsigned int j=0; j<m_height; ++j )
        {
            if( (0 == j) && (0 == i) )                          // South-West corner
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( NULL );
                m_Squares[i][j].setWest( NULL );
            }
            else if( (m_height-1 == j) && (0 == i) )            // North-West corner
            {
                m_Squares[i][j].setNorth( NULL );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( NULL );
            }
            else if( (0 == j) && (m_width-1 == i) )             // South-East corner
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( NULL );
                m_Squares[i][j].setSouth( NULL );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( (m_height-1 == j) && (m_width-1 == i) )    // North-East corner
            {
                m_Squares[i][j].setNorth( NULL );
                m_Squares[i][j].setEast( NULL );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( 0 == i )                                   // West border
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i-1][j] );
                m_Squares[i][j].setWest( NULL );
            }
            else if( 0 == j )                                   // South border
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( NULL );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( m_width-1 == i )                           // East border
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( NULL );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else if( m_height-1 == j )                          // North border
            {
                m_Squares[i][j].setNorth( NULL );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
            else
            {
                m_Squares[i][j].setNorth( &m_Squares[i][j+1] );
                m_Squares[i][j].setEast( &m_Squares[i+1][j] );
                m_Squares[i][j].setSouth( &m_Squares[i][j-1] );
                m_Squares[i][j].setWest( &m_Squares[i-1][j] );
            }
        }
    }
}

Arena::~Arena(void)
{
    m_Squares.clear();
}

void Arena::GetRandomPosition(Square * * square)
{
    ::srand( (unsigned)time( NULL ) );

    PositionType position;

    position.first = (XType)(::rand() % (m_width - 2) + 1);
    position.second = (YType)(::rand() % (m_height - 2) + 1);

    *square = &m_Squares[position.first][position.second];
}
