//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: RobotProperties.cpp
//
// Description: Server-side component that keeps track of the properties
//              of a Robot
//
//*****************************************************************************

#include "robotproperties.h"
#include "Square.h"

/////////////////////////////////////////////////////////////////////////////////////
// Static variables used to check if the sequence of the function calls is correct
static bool s_checkPreConditionsCalled = false;
static bool s_doTurnCalled = false;

/////////////////////////////////////////////////////////////////////////////////////
// RobotProperties

RobotProperties::RobotProperties(IDType id, AttackType a, DefenseType d, FoodType f)
: m_hasTurn( false )
, m_hadTurn( false )
, m_registered( false )
, m_ID( id )
, m_position( NULL )
, m_attack( a )
, m_defense( d )
, m_food( f )
, m_life( 100 )
, m_wallHit( false )
{
    m_heading = North;
}

RobotProperties::~RobotProperties(void)
{
}

bool RobotProperties::checkPreConditions(void)
{
    // Reset static check variables
    s_checkPreConditionsCalled = false;
    s_doTurnCalled = false;

    if( (false == m_hasTurn) ||   // Does the robot have a turn?
        (0 > m_life) ||           // Is the robot still alive?
        (false == m_registered) )
    {
        return( false );
    }

    s_checkPreConditionsCalled = true;

    return( true );
}

bool RobotProperties::doTurn(idleCommand const & idle)
{
    // Check sequence of function calls
    if( !s_checkPreConditionsCalled )
    {
        return( false );
    }

    // Do your thing here;
    // which is actually idling.

    s_doTurnCalled = true;

    return( true );
}

bool RobotProperties::doTurn(moveForwardCommand const & mf)
{
    // Check sequence of function calls
    if( !s_checkPreConditionsCalled )
    {
        return( false );
    }

    Square * neighbour = NULL;

    switch( m_heading )
    {
    case North:
        neighbour = m_position->getNorth();
        break;
    case East:
        neighbour = m_position->getEast();
        break;
    case South:
        neighbour = m_position->getSouth();
        break;
    case West:
        neighbour = m_position->getWest();
        break;
    }

    if( NULL != neighbour )
    {
        m_position = neighbour;
    }
    else
    {
        // We are hitting an outside wall
    }

    s_doTurnCalled = true;

    return( true );
}

bool RobotProperties::checkPostConditions(void)
{
    // Check sequence of function calls
    if( !s_checkPreConditionsCalled || !s_doTurnCalled )
    {
        return( false );
    }

    m_hasTurn = false;
    m_hadTurn = true;

    //*********************************************
    //     check some things like:
    //     - did the robot hit a wall?
    //     - is the robot under fire?
    //     - is the robot eating?
    //     - ...
    //*********************************************

    // Have we hit a wall?
    if( Square::Wall == m_position->getType() )
    {
        m_wallHit = true;
    }
    else
    {
        m_wallHit = false;
    }

    return( true );
}
