//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: RobotProperties.h
//
// Description: Server-side component that keeps track of the properties
//              of a Robot
//
//*****************************************************************************

#pragma once

#include "../CodeWarCommon/CodeWarTypes.h"

struct idleCommand;
struct moveForwardCommand;
class Square;

// The properties of the robots that are registered will be kept using
// this class.  Since every SlaveThread handles one robot, every SlaveThread
// will have one RobotProperties instance.
class RobotProperties
{
public:
    RobotProperties(IDType id=-1, AttackType a=-1, DefenseType d=-1, FoodType f=-1);
    ~RobotProperties(void);

    // Must be called before a doTurn() function call
    bool checkPreConditions(void);

    // Adjust the properties
    bool doTurn(idleCommand const & idle);
    bool doTurn(moveForwardCommand const & mf);

    // Must be called after a doTurn() function call
    bool checkPostConditions(void);

public:
    // Control properties
    bool m_hasTurn;             // The server is expecting a command
    bool m_hadTurn;             // The server has received a command
    bool m_registered;

    // Personal properties
    IDType m_ID;
    Square * m_position;
    HeadingType m_heading;
    AttackType m_attack;
    DefenseType m_defense;
    FoodType m_food;
    LifeType m_life;

    // Event properties
    WallHitType m_wallHit;
};
